// test harness program for tftpserver code that allows it to be
// tested on a normally hosted linux system

#include "tftpsrv_harness.h"
#include "tftpsrv.h"
#include <assert.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <sys/select.h>


#define min(a,b) ((a)<(b) ? (a) : (b))

void hexDump(void *buf, unsigned len)
{
  uchar *p = (uchar*) buf;
  unsigned i;
  unsigned n;
  unsigned offset=0;

  while (len)
    {
      printf("%04x: ", offset);

      n = min(16,len);

      for (i=0; i<n; ++i)
        {
          printf("%02x ",p[i]);
          if (i==8)
            printf(" ");
        }

      for (; i<16; ++i)
        {
          printf("   ");
          if (i==8)
            printf(" ");
        }

      printf(" ");

      for (i=0; i<n; ++i)
        {
          if (isgraph(p[i]) && !isspace(p[i]))
            printf("%c",p[i]);
          else
            printf(".");
        }
      printf("\n");

      len -= n;
      p += n;
    }
}

void  run_command(char *cmd, int unused)
{
  printf("run_command(%s,%d)\n",cmd,unused);
}

static uchar rxBuffer[4096];
static uchar txBuffer[4096];

uchar *NetTxPacket = txBuffer;

int NetLoop(proto_t proto)
{
  return 0;
}

rxhand_f *rxHandler;
thand_f  *timeoutHandler;
unsigned timeoutLength = 0;

void NetSetHandler(rxhand_f * f)
{
  rxHandler = f;
}

void NetSetTimeout(ulong iv, thand_f * f)
{
  timeoutHandler = f;
  timeoutLength = iv;
}

int NetState;

extern void NetStartAgain(void)
{
}

static struct sockaddr_in myaddr, otheraddr;
static int sock;

void NetSendUDPPacket(uchar *ether, IPaddr_t dest, int dport, int sport, int len)
{
  ssize_t txlen;
  assert(sock);
  //printf("Harness: NetSendUDPPacket from :%d to %08lx:%d len %d\n",sport,dest,dport,len);
  //hexDump(ether,len);
  otheraddr.sin_port = htons(dport);
  otheraddr.sin_addr.s_addr = htonl(dest);
  txlen = sendto(sock, NetTxPacket + NetEthHdrSize() + IP_HDR_SIZE, len, 0, (struct sockaddr*)&otheraddr, sizeof otheraddr);
  if (txlen < 0)
    perror("sendto");
}

// simulate calls made from UBoot NetLoop()

int main(int argc, char *argv[])
{
  int rxlen;
  socklen_t slen;
  fd_set rfds;
  struct timeval tv;
  int retval;
  
  if ((sock=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
    {
      perror("socket");
      return 1;
    }

  memset(&myaddr, 0, sizeof(myaddr));
  myaddr.sin_family = AF_INET;
  myaddr.sin_port = htons(9969);
  myaddr.sin_addr.s_addr = htonl(INADDR_ANY);
  
  if (bind(sock, (const struct sockaddr *)&myaddr, sizeof(myaddr)))
    {
      perror("bind");
      return 2;
    }

  NetState = NETLOOP_CONTINUE;
  TftpServerStart();

  while (NetState == NETLOOP_CONTINUE)
    {
      assert(rxHandler);
      assert(timeoutHandler);
      slen = sizeof(otheraddr);
      uchar *buf = ((uchar*)&rxBuffer)+NetEthHdrSize()+IP_HDR_SIZE;
      unsigned bufsize = (sizeof rxBuffer)-(NetEthHdrSize()+IP_HDR_SIZE);
      IP_t *ip = (IP_t*)(buf-IP_HDR_SIZE);

      FD_ZERO(&rfds);
      FD_SET(sock,&rfds);
      tv.tv_sec = timeoutLength / CFG_HZ;
      tv.tv_usec = ((timeoutLength * 1000000)/CFG_HZ) % 1000000;

      retval = select(sock+1, &rfds, NULL, NULL, &tv);

      if (retval < 0)
        {
          perror("select");
          return 3;
        }
      
      if (retval == 0)
        {
          //timeout
          timeoutHandler();
          continue;
        }

      if ((rxlen = recvfrom(sock, buf, bufsize, 0, (struct sockaddr *)&otheraddr, &slen)) < 0)
        {
          perror("recvfrom");
          return 4;
        }
      
      //printf("Harness: rx packet from %s:%d len %d\n", inet_ntoa(otheraddr.sin_addr), ntohs(otheraddr.sin_port), rxlen);
      //hexDump(buf,rxlen);

      // fake in the IP header fields needed by tftp server
      ip->ip_src = ntohl(otheraddr.sin_addr.s_addr);
      ip->ip_p = IPPROTO_UDP;
      ip->udp_dst = ntohs(myaddr.sin_port);
      ip->udp_src = ntohs(otheraddr.sin_port);
      // fake in the source MAC address
      rxBuffer[6] = 1;
      rxBuffer[7] = 2;
      rxBuffer[8] = 3;
      rxBuffer[9] = 4;
      rxBuffer[10] = 5;
      rxBuffer[11] = 6;

      // call handler
      rxHandler(buf,ntohs(myaddr.sin_port),ntohs(otheraddr.sin_port),rxlen);
    }
  return 0;
}


